'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let joinMonster = (() => {
  var _ref = _asyncToGenerator(function* (resolveInfo, context, dbCall, options = {}) {
    const sqlAST = (0, _queryASTToSqlAST.queryASTToSqlAST)(resolveInfo, options);
    const { sql, shapeDefinition } = yield compileSqlAST(sqlAST, context, options);
    if (!sql) return Promise.resolve({});

    return handleUserDbCall(dbCall, sql, shapeDefinition, sqlAST);
  });

  return function joinMonster(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
})();

let compileSqlAST = (() => {
  var _ref2 = _asyncToGenerator(function* (sqlAST, context, options) {
    debug((0, _util3.emphasize)('SQL_AST'), (0, _util3.inspect)(sqlAST));

    const dialect = options.dialect || 'standard';
    const stringify = require('./stringifiers/' + dialect).default;
    const sql = yield stringify(sqlAST, context);
    debug((0, _util3.emphasize)('SQL'), (0, _util3.inspect)(sql));

    const shapeDefinition = (0, _defineObjectShape2.default)(sqlAST);
    debug((0, _util3.emphasize)('SHAPE_DEFINITION'), (0, _util3.inspect)(shapeDefinition));
    return { sql, shapeDefinition };
  });

  return function compileSqlAST(_x5, _x6, _x7) {
    return _ref2.apply(this, arguments);
  };
})();

let getNode = (() => {
  var _ref3 = _asyncToGenerator(function* (typeName, resolveInfo, context, where, dbCall, options = {}) {
    const type = resolveInfo.schema.getType(typeName);

    const fakeParentNode = {
      _fields: {
        node: {
          type,
          name: type.name.toLowerCase(),
          where
        }
      }
    };
    const namespace = new _aliasNamespace2.default(options.minify);
    const sqlAST = {};
    const fieldNodes = resolveInfo.fieldNodes || resolveInfo.fieldASTs;

    (0, _queryASTToSqlAST.getGraphQLType)(fieldNodes[0], fakeParentNode, sqlAST, resolveInfo.fragments, resolveInfo.variableValues, namespace, options);
    (0, _queryASTToSqlAST.pruneDuplicateSqlDeps)(sqlAST, namespace);
    const { sql, shapeDefinition } = yield compileSqlAST(sqlAST, context, options);
    return handleUserDbCall(dbCall, sql, shapeDefinition, sqlAST).then(function (obj) {
      if (!obj) {
        return obj;
      }

      obj.__type__ = type;
      return obj;
    });
  });

  return function getNode(_x8, _x9, _x10, _x11, _x12, _x13) {
    return _ref3.apply(this, arguments);
  };
})();

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _nesthydrationjs = require('nesthydrationjs');

var _queryASTToSqlAST = require('./queryASTToSqlAST');

var _defineObjectShape = require('./defineObjectShape');

var _defineObjectShape2 = _interopRequireDefault(_defineObjectShape);

var _postProcess = require('./postProcess');

var _postProcess2 = _interopRequireDefault(_postProcess);

var _aliasNamespace = require('./aliasNamespace');

var _aliasNamespace2 = _interopRequireDefault(_aliasNamespace);

var _util3 = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const debug = require('debug')('join-monster');

joinMonster.getNode = getNode;

function handleUserDbCall(dbCall, sql, shapeDefinition, sqlAST) {
  if (dbCall.length === 2) {
    return new Promise((resolve, reject) => {
      dbCall(sql, (err, rows) => {
        if (err) {
          reject(err);
        } else {
          rows = validate(rows);
          debug((0, _util3.emphasize)('RAW_DATA'), (0, _util3.inspect)(rows.slice(0, 8)));
          debug(`${ rows.length } rows...`);
          const nested = (0, _nesthydrationjs.nest)(rows, shapeDefinition);
          resolve((0, _postProcess2.default)(nested, sqlAST));
        }
      });
    });
  }

  const result = dbCall(sql);

  if (typeof result.then === 'function') {
    return result.then(rows => {
      rows = validate(rows);
      debug((0, _util3.emphasize)('RAW DATA'), (0, _util3.inspect)(rows.slice(0, 8)));
      debug(`${ rows.length } rows...`);
      const nested = (0, _nesthydrationjs.nest)(rows, shapeDefinition);
      return (0, _postProcess2.default)(nested, sqlAST);
    });
  } else {
    return Promise.resolve((0, _nesthydrationjs.nest)(validate(result), shapeDefinition));
  }
}

function validate(rows) {
  if (Array.isArray(rows)) return rows;else if (rows && rows.rows) return rows.rows;else {
      throw new Error(`"dbCall" function must return/resolve an array of objects where each object is a row from the result set. Instead got ${ _util2.default.inspect(rows, { depth: 3 }) }`);
    }
}

joinMonster.version = require('../package.json').version;
exports.default = joinMonster;